#
# core.py
#
# Copyright (C) 2009 Szentandrasi Istvan <szentandrasii@gmail.com>
#
# Basic plugin template created by:
# Copyright (C) 2008 Martijn Voncken <mvoncken@gmail.com>
# Copyright (C) 2007-2009 Andrew Resch <andrewresch@gmail.com>
# Copyright (C) 2009 Damien Churchill <damoxc@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

from __future__ import unicode_literals


import logging
log = logging.getLogger(__name__)

from deluge.plugins.pluginbase import CorePluginBase
import deluge.component as component
import deluge.configmanager
from deluge.core.rpcserver import export
from twisted.internet.task import LoopingCall


DEFAULT_PREFS = {
    "totalup": 0,
    "totaldown": 0,
    "ptotalup": 0,
    "ptotaldown": 0,
    "showPayload": False,
    "showTotal": False
}

class Core(CorePluginBase):
    def enable(self):
        self.config = deluge.configmanager.ConfigManager("totaltraffic.conf", DEFAULT_PREFS)
        self.prev_total_upload = self.config['totalup']
        self.prev_total_download = self.config['totaldown']
        self.prev_ptotal_upload = self.config['ptotalup']
        self.prev_ptotal_download = self.config['ptotaldown']

        self.calc_stats()

        #add timers
        self.update_status_timer = LoopingCall(self.calc_stats)
        self.update_status_timer.start(2)
        self.writeback_timer = LoopingCall(self.writeback)
        self.writeback_timer.start(60)

    def disable(self):
        self.update_status_timer.stop()
        self.writeback_timer.stop()
        self.writeback()

    def update(self):
        pass

    def writeback(self):
        log.info('TotalTraffic: writeback')
        self.config['totalup'] = self.total_upload
        self.config['totaldown'] = self.total_download
        self.config['ptotalup'] = self.ptotal_upload
        self.config['ptotaldown'] = self.ptotal_download
        self.config.save()


    def calc_stats(self):
        status = component.get("Core").get_session_status(["total_download", "total_upload", "total_payload_download", "total_payload_upload"])
        self.total_upload = self.prev_total_upload + status["total_upload"]
        self.total_download = self.prev_total_download + status["total_download"]
        self.ptotal_upload = self.prev_ptotal_upload + status["total_payload_upload"]
        self.ptotal_download = self.prev_ptotal_download + status["total_payload_download"]
        self.down = status["total_download"]
        self.up = status["total_upload"]
        self.payload_up = status["total_payload_upload"]
        self.payload_down = status["total_payload_download"]

    @export
    def get_status(self):
        if self.config['showPayload']:
            traff = {'up': self.payload_up,'down': self.payload_down,
                    't_up': self.ptotal_upload, 't_down': self.ptotal_download}
        else:
            traff = { 'up': self.up,'down': self.down,
                    't_up': self.total_upload, 't_down': self.total_download}
        return {'showTotal': self.config['showTotal'], 'traffic': traff}
    
    @export
    def reset_timers(self):
        self.config['totalup'] = 0
        self.config['totaldown'] = 0
        self.config['ptotalup'] = 0
        self.config['ptotaldown'] = 0
        self.prev_total_upload = 0
        self.prev_total_download = 0
        self.prev_ptotal_upload = 0
        self.prev_ptotal_download = 0
        self.config.save()

    @export
    def set_config(self, config):
        "sets the config dictionary"
        for key in config.keys():
            self.config[key] = config[key]
        self.config.save()

    @export
    def get_config(self):
        "returns the config dictionary"
        return self.config.config
